"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
const model = require("./model");
exports.MINIMAL = 'MINIMAL';
exports.MAXIMAL = 'MAXIMAL';
let DataLevelModel = class DataLevelModel extends model.Model {
    toJSON(options) {
        let obj = this.toObject(options);
        if (options && options.level) {
            const fields = (this.schema.dataLevel && this.schema.dataLevel.levelMap[options.level]
                || []);
            obj = _.pick(obj, fields);
        }
        return obj;
    }
};
DataLevelModel = __decorate([
    model.Plugin({
        fn: dataLevelPlugin,
        priority: 100,
    })
], DataLevelModel);
exports.DataLevelModel = DataLevelModel;
function dataLevelPlugin(schema, options) {
    if (schema.dataLevel == null) {
        schema.dataLevel = {
            levelMap: {},
        };
    }
    // schema.levels = [ MINIMAL ].concat(options.levels || []);
    addToLevelMap(schema, levelPaths(schema));
    for (const name of model.preQueries) {
        schema.pre(name, modifyProjection);
    }
}
function modifyProjection(next) {
    const schema = this.schema;
    const level = (this.options.level ||
        schema.options.dataLevel && schema.options.dataLevel.default);
    if (level) {
        const fields = schema.dataLevel.levelMap[level] || [];
        if (this._fields == null) {
            this._fields = {};
        }
        for (const field of fields) {
            this._fields[field] = 1;
        }
    }
    next();
}
function addToLevelMap(schema, lps) {
    if (schema.parentSchema) {
        addToLevelMap(schema.parentSchema, lps);
    }
    const dataLevelOptionsLevels = _.values(schema.options.dataLevel && schema.options.dataLevel.levels || []);
    const levelMap = schema.dataLevel.levelMap;
    for (const { path, level } of lps) {
        for (let levelIndex = (level === exports.MINIMAL ? 0 : dataLevelOptionsLevels.indexOf(level)); levelIndex >= 0 && levelIndex < dataLevelOptionsLevels.length; levelIndex++) {
            const cLevel = dataLevelOptionsLevels[levelIndex];
            levelMap[cLevel] = _.union(levelMap[cLevel], [path]);
        }
        if (level === exports.MINIMAL) {
            levelMap[exports.MINIMAL] = _.union(levelMap[exports.MINIMAL], [path]);
        }
        levelMap[exports.MAXIMAL] = _.union(levelMap[exports.MAXIMAL], [path]);
    }
}
function levelPaths(schema) {
    const res = [];
    schema.eachPath((pathname, schemaType) => {
        res.push({
            path: pathname,
            level: schemaType.options.level || exports.MINIMAL,
        });
    });
    return res;
}

//# sourceMappingURL=data-level.js.map
